VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsFile"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'Author: Zbigniew Borysiuk
'E-mail: eebr@earthlink.net
'The following code demonstrates how to create a custom resource file.
'It is published for demonstration purpose only.
'
'The clsFile object exposes simple interface that allows to:
'- create and delete a resource file
'- add, retrieve, and delete files from the resource file
'- retrieve a file from a database field and then save it in the repository
'- retrieve a file from the repository  and then save it in a database field
'- handle duplicate files
'- get information about the content of the repository

' The TestFile project is a test project that demonstrates the possible
' use of the clsFile object.

'Project > References > "Microsoft ActiveX data object 2.0 library"

'****************************************************************************
Option Explicit
'notifies that the last file is about to be deleted
Public Event DeleteOneFile(ByRef blnFlag As Boolean)
'notifies that a file has been deleted
Public Event FileDeleted()
'notifies that the repository has been deleted
Public Event RepositoryDeleted()
'notifies that the file name has been truncated
Public Event NameTruncated(ByRef strName As String)
'notifies that a duplicate file exists in the repository
Public Event DupExists(ByRef blnCancel As Boolean)
'notifies that a file with the same name exists in the location
'specified by a client
Public Event FExists(ByRef blnCancel As Boolean)
'notifies that some of the temp files might have not
'been put to end
Public Event FileDeletedWithErr(ByVal strFile1 As String, ByVal strFile2 As String)

'storage for the Status property
Private m_blnStatus As Boolean
'storage for the LastErr property
Private m_strLastErr As String
'storage for the Content property
Private m_varContent()

'holds a fully qualified name of a file containing
'information about the repository
Private m_strMetaFile As String
'holds a fully qualified name of the repository
Private m_strRepFile As String


'used as an argument of the GetProperty
'function
Private Enum RepProperty
    brSize
    brNum
End Enum
'used as the return value of the
'DeleteFile function
Public Enum BinaryRepDeleteEnum
    brDeleteSuccess
    brDeleteCanceledOneFile
    brDeleteCanceledEmptyRep
    brDeleteCanceledFileNoExist
    brDeleteError
    brDeleteRepositorySuccess
    brDeleteClosingError
End Enum


'this structure describes the repository
Private Type InfoBinaryFile
'the number of the files stored in the repository
    intNumFiles As Integer
'the size of the repository
    lngFileSize As Long
End Type

'this structure describes each
'file stored in the repository
Private Type InfoSubFile
' the size of the file
    lngFileSize As Long
'the start of the file
    lngFileStart As Long
'the name of the file
    strFileName As String * 20
End Type

Const EDGE As Long = 16384
Const CHUNK As Long = 16384

Const INVALID_HANDLE_VALUE = -1
Const MAX_PATH = 260

Private Type FILETIME
    dwLowDateTime As Long
    dwHighDateTime As Long
End Type
Private Type WIN32_FIND_DATA
    dwFileAttributes As Long
    ftCreationTime As FILETIME
    ftLastAccessTime As FILETIME
    ftLastWriteTime As FILETIME
    nFileSizeHigh As Long
    nFileSizeLow As Long
    dwReserved0 As Long
    dwReserved1 As Long
    cFileName As String * MAX_PATH
    cAlternate As String * 14
End Type
Private Declare Function FindFirstFile Lib "kernel32" _
Alias "FindFirstFileA" (ByVal lpFileName As String, _
lpFindFileData As WIN32_FIND_DATA) As Long
Private Declare Function FindClose Lib "kernel32" _
(ByVal hFindFile As Long) As Long
Private Declare Function GetTempPath Lib "kernel32" _
Alias "GetTempPathA" (ByVal nBufferLength As Long, _
ByVal lpBuffer As String) As Long
Private Declare Function GetTempFileName Lib "kernel32" _
Alias "GetTempFileNameA" (ByVal lpszPath As String, _
ByVal lpPrefixString As String, ByVal wUnique As Long, _
ByVal lpTempFileName As String) As Long
Public Function WriteFile(ByVal strFile As String) As Boolean
'inserts a file into the repository.
On Error GoTo ErrHandler
If Not FileExists(strFile) Then
    Err.Raise vbObjectError + 2003, , "Component can't find file: " & strFile
End If
'call AddFile to put the file into the repository
If AddFile(strFile) Then
    WriteFile = True
Else
    WriteFile = False
End If
Exit Function
ErrHandler:
m_strLastErr = "Error #" & Err.Number & "; prjTransferData.clsFile;" & Err.Description
WriteFile = False
End Function

Public Function GetFile(ByVal strFile As String) As Byte()
'The GetFile function retrieves data from the resource file.
Dim i As Integer
Dim intFile As Integer
Dim infobinary As InfoBinaryFile
Dim SubFile() As InfoSubFile
Dim bytData() As Byte
On Error GoTo ErrGetFile
If strFile = "" Then
    Err.Raise vbObjectError + 2008, , "Missing file name."
End If
If Not FileExists(m_strRepFile) Then
    Err.Raise vbObjectError + 2000, , "Component can't find " & m_strRepFile & "."
End If

If Not FileExists(m_strMetaFile) Then
    Err.Raise vbObjectError + 2001, , "Component can't find " & m_strMetaFile & "."
End If
intFile = FreeFile
Open m_strMetaFile For Binary Access Read Lock Write As intFile

Get intFile, 1, infobinary
'check integrity of the data
If Not ValidateMetaFile(infobinary.intNumFiles, intFile) Then
    Err.Raise vbObjectError + 2004, , "The data are corrupted."
End If

If Not ValidateFile(infobinary) Then
    Err.Raise vbObjectError + 2002, , "The repository is corrupted."
End If

ReDim SubFile(infobinary.intNumFiles - 1)

Get intFile, , SubFile
Close intFile
strFile = GetFileNameB(strFile)
'read the data
For i = 0 To UBound(SubFile)
    If Trim(strFile) = Trim(SubFile(i).strFileName) Then
        intFile = FreeFile
        Open m_strRepFile For Binary As intFile
        ReDim bytData(SubFile(i).lngFileSize - 1)
        Get intFile, SubFile(i).lngFileStart, bytData
        Close intFile
        GetFile = bytData
        Exit Function
    End If
Next
ReDim GetFile(0)

Exit Function
ErrGetFile:
m_strLastErr = "Error " & Err.Number & "; prjTransferData,clsFile; " & Err.Description

If intFile <> 0 Then Close intFile
ReDim GetFile(0)


End Function
Private Function ValidateFile(infobinary As InfoBinaryFile) As Boolean
'checks integrity of the data.
If FileLen(m_strRepFile) <> infobinary.lngFileSize Then
    ValidateFile = False
Else
    ValidateFile = True
End If
End Function
Private Sub OpenFile()
' creates the repository.
Dim intFileMeta As Integer
Dim intFileRep As Integer
Dim blnMF As Boolean
Dim blnRP As Boolean
On Error GoTo ErrHandler

intFileMeta = FreeFile
Open m_strMetaFile For Binary Access Read Write As intFileMeta
blnMF = True
Close intFileMeta
intFileRep = FreeFile
Open m_strRepFile For Binary As intFileRep
blnRP = True
Close intFileRep
Exit Sub
ErrHandler:
If intFileMeta <> 0 Then Close intFileMeta
If intFileRep <> 0 Then Close intFileRep
If blnMF Then Kill m_strMetaFile
If blnRP Then Kill m_strRepFile
Err.Raise Err.Number
End Sub
Private Function FileExists(ByVal strFile As String) As Boolean
'searches for a file whose name matches
'the file name passed as an argument.

Dim WFD As WIN32_FIND_DATA
Dim hSearch As Long
Dim rtn As Long
hSearch = FindFirstFile(strFile, WFD)
 If hSearch <> INVALID_HANDLE_VALUE Then
    FileExists = True
 Else
    FileExists = False
 End If
rtn = FindClose(hSearch)

End Function

Private Sub Class_Initialize()
On Error GoTo ErrHandler
m_strMetaFile = GetSetting("prjTransferData", "Startup", "Path1", "Not Set")
m_strRepFile = GetSetting("prjTransferData", "Startup", "Path2", "Not Set")
'check if the resource file exists
If m_strMetaFile <> "Not Set" And m_strRepFile <> "Not Set" Then
    If Not FileExists(m_strMetaFile) Or Not FileExists(m_strRepFile) Then
        m_blnStatus = False
    Else
        m_blnStatus = True

    End If

End If
Exit Sub
ErrHandler:
m_strLastErr = "Error " & Err.Number & "; prjTransferData.clsFile; " _
    & vbCrLf & Err.Description & " during intialization."
End Sub
Public Function KillFile() As Boolean
'deletes the repository.
On Error GoTo ErrHandler
Kill m_strRepFile
Kill m_strMetaFile
DeleteSetting "prjTransferData", "Startup", "Path1"
DeleteSetting "prjTransferData", "Startup", "Path2"
RaiseEvent RepositoryDeleted
m_blnStatus = False
KillFile = True
Exit Function
ErrHandler:
KillFile = False
m_strLastErr = "Error #" & Err.Number & "; prjTransferData.clsFile; " & Err.Description
End Function
Public Function CopyFile(ByVal strFile As String, Optional strDestination As String = "") As Boolean
'The ExtractFile function copies a file stored in the repository to a new file.
Dim bytData() As Byte
Dim strNewLocation As String
Dim intFile As Integer
Dim blnCancel As Boolean
On Error GoTo ErrExtract
'retrieve the data from the repository
bytData = GetFile(strFile)
If UBound(bytData) = 0 Then
    CopyFile = False
    Exit Function
End If
'set the new location
If strDestination = "" Then
    strFile = GetFileNameB(strFile)
    strNewLocation = "c:\" & strFile
Else
    strNewLocation = strDestination
End If
If FileExists(strNewLocation) Then
'a file with the same name found in the new location
        RaiseEvent FExists(blnCancel) 'allow a client to take appropriate action
            If blnCancel Then
                CopyFile = False
                Exit Function
            End If
        Kill strNewLocation
End If

intFile = FreeFile
Open strNewLocation For Binary Access Write Lock Read As intFile
Put intFile, , bytData
Close intFile
CopyFile = True
Exit Function
ErrExtract:
m_strLastErr = "Error #" & Err.Number & "; prjTransferData.clsFile; " & Err.Description
If intFile <> 0 Then Close intFile
CopyFile = False

End Function
Public Function ExtractFromField(Field As ADODB.Field, ByVal strFName As String) As Boolean
'The ExtractFromField function retrieves a file from a database field
'and then stores it in the resource file.

Dim strFile As String
Dim lngFSize As Long
Dim blnFlag As Boolean
On Error GoTo ErrExtractField

If strFName = "" Then
    Err.Raise vbObjectError + 2008, , "Missing file name."
End If

If Field.Type <> adLongVarBinary Then
    Err.Raise vbObjectError + 2007, , "Only adLongVarBinary is supported."
End If

strFName = GetFileNameB(strFName)
'get the size of the file
lngFSize = Field.ActualSize
'determine the way of copying the data
If lngFSize > EDGE Then
'in chunks
    strFile = ExtractInChunks(Field, lngFSize, strFName)
Else
'all
    strFile = ExtractAll(Field, strFName)
End If

If strFile = "" Then
    ExtractFromField = False 'error
    Exit Function
End If

'store the file in the repository
If AddFile(strFile) Then
    ExtractFromField = True
Else
    ExtractFromField = False
End If
100  Kill strFile

ExitExtractFromField:
    Exit Function

ErrExtractField:
m_strLastErr = "Error #" & Err.Number & "; prjTransferData.clsFile; " & Err.Description
If Erl = 100 Then
  Resume ExitExtractFromField
End If
ExtractFromField = False

End Function
Private Function ExtractInChunks(Field As ADODB.Field, _
                          ByVal FSize As Long, strFName As String) As String
' retrieves the chunks of the data, and then saves the data in a temporary file.

Dim Data() As Byte, BytesRead As Long, intFN As Integer, strFile As String, _
lngOffset As Long
  On Error GoTo ErrHandler
       strFile = GetTempFile(strFName)
       intFN = FreeFile
       'open the temporary file
        Open strFile For Binary As intFN
    'get the remainder
        lngOffset = FSize Mod CHUNK
        Do While FSize <> BytesRead
          If lngOffset <> 0 Then
            Data = Field.GetChunk(lngOffset)
            BytesRead = lngOffset
            lngOffset = 0
          Else
            Data = Field.GetChunk(CHUNK)
            BytesRead = BytesRead + CHUNK
          End If
          Put intFN, , Data
        Loop
     Close intFN
     'we are happy
     ExtractInChunks = strFile
Exit Function
ErrHandler:
m_strLastErr = "Error #" & Err.Number & "; prjTransferData.clsFile; " & Err.Description
If intFN <> 0 Then Close intFN
ExtractInChunks = ""

End Function

Private Function GetTempFile(Optional strName As String) As String
' finds a unique name for a temp file and - optionally - creates it.
Dim strPath As String, lngLen As Long, intBuffer As Integer, _
strTempName As String, rtn As Long
intBuffer = 256
strPath = String$(intBuffer, Chr$(0))
'find the path of the directory designated for temporary files
lngLen = GetTempPath(intBuffer, strPath)
If lngLen = 0 Then
    strPath = "c:\"
End If
'remove the rest of the buffer
strPath = Left$(strPath, lngLen)
If strName = "" Then
    strTempName = String$(intBuffer, Chr$(0))
'find a unique file name, and then create the file.
    rtn = GetTempFileName(strPath, ".tmp", 0, strTempName)
    If rtn = 0 Then
        Err.Raise vbObjectError + 2009, , "Creating a temporary file failed."
    End If
    strTempName = Left$(strTempName, InStr(1, strTempName, Chr$(0)) - 1)
Else
    strTempName = strPath & strName
    If FileExists(strTempName) Then
        Kill strTempName
    End If
End If
GetTempFile = strTempName
End Function

Private Function ExtractAll(fld As ADODB.Field, ByVal strFile As String) As String
' retrieves all of the  data, and then saves the data in the temporary file.

Dim bytData() As Byte, intFN As Integer
On Error GoTo ErrHandler
strFile = GetTempFile(strFile)
bytData = fld.Value
intFN = FreeFile
Open strFile For Binary As intFN
Put intFN, , bytData
Close intFN
ExtractAll = strFile
Exit Function
ErrHandler:
m_strLastErr = "Error #" & Err.Number & "; prjTransferData.clsFile; " & Err.Description
If intFN <> 0 Then Close intFN
ExtractAll = ""
End Function

Public Function StoreInField(Field As ADODB.Field, ByVal strFile As String) As Boolean
'retrieves a file from the repository, and then saves it in the database field.
Dim v
Dim blnRtn As Boolean
On Error GoTo ErrHandler
If strFile = "" Then
    Err.Raise vbObjectError + 2008, , "Missing file name."
End If

If Field.Type <> adLongVarBinary Then
    Err.Raise vbObjectError + 2007, , "Only adLongVarBinary is supported."
End If

strFile = GetFileNameB(strFile)
'get the size and the start of the file
v = FileSize(strFile)
If UBound(v) = 0 Then
    StoreInField = False
    Exit Function
End If

If v(0) > EDGE Then
    blnRtn = SaveInChunks(Field, v(0), v(1)) 'copy in chunks
Else
    blnRtn = SaveAll(Field, v(0), v(1)) 'copy all
End If
           
If Not blnRtn Then
    StoreInField = False
Else
    StoreInField = True
End If
Exit Function
ErrHandler:
m_strLastErr = "Error #" & Err.Number & "; prjTransferData.clsFile; " & Err.Description
StoreInField = False

End Function
Private Function FileSize(ByVal strFile As String) As Long()
'finds the size and the start of a file.
Dim i As Integer
Dim intFile As Integer
Dim infobinary As InfoBinaryFile
Dim SubFile() As InfoSubFile
Dim l(1) As Long

On Error GoTo ErrHandler
If Not FileExists(m_strMetaFile) Then
    Err.Raise vbObjectError + 2001, , "Component can't find " & m_strMetaFile & "."
End If

intFile = FreeFile
Open m_strMetaFile For Binary Access Read Write As intFile
Get intFile, 1, infobinary
If Not ValidateMetaFile(infobinary.intNumFiles, intFile) Then
    Err.Raise vbObjectError + 2004, , "The data are corrupted."
End If

If Not ValidateFile(infobinary) Then
   Err.Raise vbObjectError + 2002, , "Repository is corrupted."
End If

ReDim SubFile(infobinary.intNumFiles - 1)
Get intFile, , SubFile
For i = 0 To UBound(SubFile)
    If Trim(strFile) = Trim(SubFile(i).strFileName) Then
        l(0) = SubFile(i).lngFileSize
        l(1) = SubFile(i).lngFileStart
        Exit For
    End If
Next
Close intFile
FileSize = l
Exit Function
ErrHandler:
m_strLastErr = "Error #" & Err.Number & "; prjTransferData.clsFile; " & Err.Description
If intFile <> 0 Then Close intFile
ReDim FileSize(0)

End Function
Private Function SaveInChunks(fld As ADODB.Field, ByVal lngFileSize As Long, _
                    ByVal FileStart As Long) As Boolean
'saves the data in the Field object.
     
Dim Data() As Byte, lngBytesRead As Long, intFN As Integer, lngOffset As Long
 
 On Error GoTo ErrHandler
lngOffset = lngFileSize Mod CHUNK

intFN = FreeFile
Open m_strRepFile For Binary As intFN

  
  Do While lngFileSize <> lngBytesRead
    If lngOffset <> 0 Then
      ReDim Data((lngOffset) - 1)
      Get intFN, FileStart, Data
      lngBytesRead = lngOffset
      lngOffset = 0
    Else
      ReDim Data(CHUNK - 1)
      Get intFN, FileStart + lngBytesRead, Data
      lngBytesRead = lngBytesRead + CHUNK
    End If
    fld.AppendChunk Data
  Loop
Close intFN
SaveInChunks = True
Exit Function
ErrHandler:
m_strLastErr = "Error #" & Err.Number & "; prjTransferData.clsFile; " & Err.Description
If intFN <> 0 Then Close intFN
SaveInChunks = False
End Function
Private Function SaveAll(fld As ADODB.Field, ByVal FileSize As Long, _
                        ByVal FileStart As Long) As Boolean
'saves the data in the Field object.
Dim Data() As Byte, BytesRead As Long, intFN As Integer
On Error GoTo ErrHandler
intFN = FreeFile
Open m_strRepFile For Binary As intFN
ReDim Data(FileSize)
Get intFN, FileStart, Data
fld.Value = Data
Close intFN
SaveAll = True
Exit Function
ErrHandler:
m_strLastErr = "Error #" & Err.Number & "; prjTransferData.clsFile; " & Err.Description
If intFN <> 0 Then Close intFN
SaveAll = False
End Function

Public Function DeleteFile(ByVal strFile As String) As BinaryRepDeleteEnum
'deletes a file from the repository.
Dim intTFN As Integer
Dim intMetaFN As Integer
Dim intFN As Integer
Dim InfoFile As InfoBinaryFile
Dim InfoSubFile() As InfoSubFile
Dim InfoSubFileNew() As InfoSubFile
Dim b() As Byte
Dim strTemp As String, strTempMeta As String
Dim i As Integer
Dim j As Integer
Dim lngNewStart As Long
Dim blnFlag As Boolean
Dim blnCancel As Boolean
Dim blnTemp As Boolean
Dim blnTempMeta As Boolean
Dim blnClosing As Boolean
Dim blnClosingErr As Boolean

On Error GoTo ErrHandler

If strFile = "" Then
    Err.Raise vbObjectError + 2008, , "Missing file name ."
End If
If Not FileExists(m_strMetaFile) Then
    Err.Raise vbObjectError + 2001, , "Component can't find " & m_strMetaFile & "."
End If

If Not FileExists(m_strRepFile) Then
    Err.Raise vbObjectError + 2000, , "Component can't find " & m_strRepFile & "."
End If
intMetaFN = FreeFile
Open m_strMetaFile For Binary As intMetaFN
Get intMetaFN, , InfoFile
If Not ValidateMetaFile(InfoFile.intNumFiles, intMetaFN) Then
    Err.Raise vbObjectError + 2004, , "The data are corrupted."
End If

If Not ValidateFile(InfoFile) Then
   Err.Raise vbObjectError + 2002, , "The repository is corrupted."
End If

ReDim InfoSubFile(InfoFile.intNumFiles - 1)
Get intMetaFN, , InfoSubFile
Close intMetaFN

'check if the file exists in the repository
If Not FileExistInRep(Trim(strFile), InfoSubFile()) Then
    DeleteFile = brDeleteCanceledFileNoExist
    Exit Function
End If


'deleting the last file results in deleting the repository
If InfoFile.intNumFiles = 1 Then
        blnFlag = True
'allow a client to take appropriate action
        RaiseEvent DeleteOneFile(blnFlag)
        
        If blnFlag = False Then            'deleting canceled
            DeleteFile = brDeleteCanceledOneFile
            Exit Function
        Else                                'delete repository
           If KillFile Then
                DeleteFile = brDeleteRepositorySuccess
                Exit Function
            Else
                DeleteFile = brDeleteError
                Exit Function
            End If
        End If
        
End If

    
InfoFile.intNumFiles = 0
InfoFile.lngFileSize = 0
strTemp = GetTempFile
intTFN = FreeFile
Open strTemp For Binary As intTFN
intFN = FreeFile
Open m_strRepFile For Binary As intFN
'copy the files into the temporary file except the one
'being deleted
For i = LBound(InfoSubFile) To UBound(InfoSubFile)
    If Trim(strFile) <> Trim(InfoSubFile(i).strFileName) Then
        ReDim Preserve InfoSubFileNew(j)
        InfoSubFileNew(j).lngFileSize = InfoSubFile(i).lngFileSize
        InfoSubFileNew(j).lngFileStart = lngNewStart + 1
        lngNewStart = (InfoSubFileNew(j).lngFileStart + InfoSubFileNew(j).lngFileSize) - 1
        InfoSubFileNew(j).strFileName = InfoSubFile(i).strFileName
        InfoFile.intNumFiles = InfoFile.intNumFiles + 1
        InfoFile.lngFileSize = InfoFile.lngFileSize + InfoSubFileNew(j).lngFileSize
        ReDim b(InfoSubFile(i).lngFileSize - 1)
        Get intFN, InfoSubFile(i).lngFileStart, b()
        Put intTFN, InfoSubFileNew(j).lngFileStart, b()
        j = j + 1
     Else
        strFile = "" 'only one file at a time can be deleted
    End If
    
Next
Close intFN
Close intTFN
strTempMeta = GetTempFile
intMetaFN = FreeFile
Open strTempMeta For Binary As intMetaFN
Put intMetaFN, , InfoFile
Put intMetaFN, , InfoSubFileNew
Close intMetaFN

'check the copy
If FileLen(strTemp) <> InfoFile.lngFileSize Then
'get rid of the copy
    Kill strTemp
    Kill strTempMeta
    DeleteFile = brDeleteError
    Exit Function
End If
'the copy is OK, cleanup
blnClosing = True
Kill m_strMetaFile
Kill m_strRepFile
ReDim InfoSubFile(0)
ReDim InfoSubFileNew(0)
'copy the files
FileCopy strTemp, m_strRepFile
FileCopy strTempMeta, m_strMetaFile
'delete the temporary files
Kill strTemp
Kill strTempMeta
    
If Not blnClosingErr Then
    'we are happy
    DeleteFile = brDeleteSuccess
    RaiseEvent FileDeleted
Else
    'error
    RaiseEvent FileDeletedWithErr(strTemp, strTempMeta)
End If

Exit Function
ErrHandler:

m_strLastErr = "Error #" & Err.Number & "; prjTransferData.clsFile; " & Err.Description
If blnClosing Then
    blnClosingErr = True
    DeleteFile = brDeleteClosingError
    Resume Next
End If
DeleteFile = brDeleteError
If intMetaFN <> 0 Then Close intMetaFN
If intFN <> 0 Then Close intFN
If intTFN <> 0 Then Close intTFN

End Function
Private Function FileExistInRep(strFile As String, InfoSubFile() As InfoSubFile) As Boolean
'checks if the file exists in the repository.
Dim i As Integer
For i = LBound(InfoSubFile) To (UBound(InfoSubFile))
If strFile = Trim(InfoSubFile(i).strFileName) Then
    FileExistInRep = True
    Exit Function
End If
Next

End Function
Public Property Get Content() As Variant()
' returns information about each file in the repository.
Dim InfoSubFile() As InfoSubFile
Dim InfoFile As InfoBinaryFile
Dim intFN As Integer
Dim i As Integer
Dim x As Integer
Dim v()
On Error GoTo ErrHandler

If Not FileExists(m_strMetaFile) Then
    Err.Raise vbObjectError + 2001, , "Component can't find " & m_strMetaFile & "."
End If
If Not FileExists(m_strRepFile) Then
    Err.Raise vbObjectError + 2000, , "Component can't find " & m_strRepFile & "."
End If

intFN = FreeFile
Open m_strMetaFile For Binary As intFN
Get intFN, , InfoFile

If Not ValidateMetaFile(InfoFile.intNumFiles, intFN) Then
    Err.Raise vbObjectError + 2004, , "The data are corrupted."
End If

If Not ValidateFile(InfoFile) Then
   Err.Raise vbObjectError + 2002, , "The repository is corrupted."
End If

If InfoFile.intNumFiles = 0 Then
    Err.Raise vbObjectError + 2011, , "The file " & m_strMetaFile _
            & " is empty."
End If


ReDim InfoSubFile(InfoFile.intNumFiles - 1)
Get intFN, , InfoSubFile()
Close intFN
ReDim v(2, InfoFile.intNumFiles - 1)
For i = 0 To UBound(InfoSubFile)
    v(0, i) = InfoSubFile(i).lngFileSize
    v(1, i) = InfoSubFile(i).lngFileStart
    v(2, i) = Trim(InfoSubFile(i).strFileName)
Next

m_varContent = v
Content = v

Exit Property
ErrHandler:
m_strLastErr = "Error #" & Err.Number & "; prjTransferData.clsFile; " & Err.Description
ReDim m_varContent(0)
 m_varContent(0) = -1
 Content = m_varContent
If intFN <> 0 Then Close intFN

End Property
Public Property Get NumFiles() As Integer
'returns the number of the files in the repository.

NumFiles = GetProperty(brNum)
End Property
Private Function GetProperty(varProperty As RepProperty) As Variant
'The GetProperty function retrieves the information about
'the size of the repository and the number of the files stored in it.

Dim InfoFile As InfoBinaryFile
Dim intFN As Integer
On Error GoTo ErrHandler
If Not FileExists(m_strMetaFile) Then
    Err.Raise vbObjectError + 2001, , "Component can't find " & m_strMetaFile & "."
End If
If Not FileExists(m_strRepFile) Then
    Err.Raise vbObjectError + 2000, , "Component can't find " & m_strRepFile & "."
End If

intFN = FreeFile
Open m_strMetaFile For Binary As intFN
Get intFN, , InfoFile

If Not ValidateMetaFile(InfoFile.intNumFiles, intFN) Then
    Err.Raise vbObjectError + 2004, , "The data are corrupted."
End If

If Not ValidateFile(InfoFile) Then
   Err.Raise vbObjectError + 2002, , "The repository is corrupted."
End If


Close intFN

Select Case varProperty
Case brNum '
GetProperty = InfoFile.intNumFiles
Case brSize
GetProperty = InfoFile.lngFileSize
End Select
Close intFN
Exit Function
ErrHandler:
m_strLastErr = "Error #" & Err.Number & "; prjTransferData.clsFile; " & Err.Description
GetProperty = -1
If intFN <> 0 Then Close intFN

End Function
Public Property Get Size() As Long
'returns the size of the repository
Size = GetProperty(brSize)
End Property
Private Function AddFile(ByVal strFile As String) As Boolean
'The AddFile function inserts a file into the repository.

Dim intFile As Integer
Dim SubFile()  As InfoSubFile
Dim infobinary As InfoBinaryFile
Dim bytByteData() As Byte
Dim intMetaFile As Integer
Dim x As Integer
Dim lngFLen As Long
Dim strTempName As String
Dim blnCancel As Boolean
On Error GoTo ErrHandler


If Not FileExists(m_strMetaFile) Then
    Err.Raise vbObjectError + 2001, , "Component can't find " & m_strMetaFile & "."
End If
If Not FileExists(m_strRepFile) Then
    Err.Raise vbObjectError + 2000, , "Component can't find " & m_strRepFile & "."
End If

intMetaFile = FreeFile
Open m_strMetaFile For Binary Access Read Write As intMetaFile
Get intMetaFile, 1, infobinary
x = infobinary.intNumFiles
If Not ValidateMetaFile(x, intMetaFile) Then
    Err.Raise vbObjectError + 2004, , "The data are corrupted."
End If

If FileLen(m_strRepFile) <> infobinary.lngFileSize Then
    Err.Raise vbObjectError + 2002, , "Repository is corrupted."
End If
 strTempName = GetFileNameB(strFile)
 lngFLen = Len(strTempName)
'the name can be only 20 characters long
If lngFLen > 20 Then
'truncate the name
    strTempName = Left$(strTempName, 20)
'allow a client to take appropriate action
    RaiseEvent NameTruncated(strTempName)
End If
If infobinary.lngFileSize = 0 Then
    ReDim SubFile(0)
Else
    ReDim Preserve SubFile(x)
    Get intMetaFile, 7, SubFile
End If


'check if the dupes exist
If FileExistInRep(strTempName, SubFile) Then
    RaiseEvent DupExists(blnCancel)
' check if the client wants to cancel inserting the duplicate file
        If blnCancel Then
            Close intMetaFile
            AddFile = False
            Exit Function
        End If
End If
intFile = FreeFile
Open strFile For Binary Access Read Lock Write As intFile
'set info about the new file
If SetInfo(intFile, SubFile(x), infobinary, strTempName) = False Then
    Close intMetaFile
    Close intFile
    AddFile = False
    Exit Function
End If
ReDim bytByteData(LOF(intFile) - 1)
Get intFile, 1, bytByteData
Close intFile
intFile = FreeFile
Open m_strRepFile For Binary Access Write Lock Read As intFile
Put intFile, SubFile(x).lngFileStart, bytByteData
Put intMetaFile, 1, infobinary
Put intMetaFile, , SubFile
Close intMetaFile
Close intFile
AddFile = True
Exit Function
ErrHandler:
m_strLastErr = "Error #" & Err.Number & "; prjTransferData.clsFile; " & Err.Description
If intMetaFile <> 0 Then Close intMetaFile
If intFile <> 0 Then Close intFile
AddFile = False

End Function

Private Function SetInfo(ByVal intFile As Integer, SubFile As InfoSubFile, infobinary As InfoBinaryFile, ByVal strTempName As String) As Boolean
'saves the data about the repository into the infobinary and SubFile variables
On Error GoTo ErrSetInfo
'the size of the new file
SubFile.lngFileSize = LOF(intFile)
'the start of the new file
SubFile.lngFileStart = infobinary.lngFileSize + 1
'the name of the new file
SubFile.strFileName = strTempName

'the number of the files stored in the repository
infobinary.intNumFiles = infobinary.intNumFiles + 1
'the size of the repository
infobinary.lngFileSize = infobinary.lngFileSize + SubFile.lngFileSize
SetInfo = True
Exit Function
ErrSetInfo:
SetInfo = False
m_strLastErr = " Error #" & Err.Number & "; prjTransferData.clsFile; " & Err.Description

End Function

Public Property Get LastErr() As String
'returns information about the last error.
LastErr = m_strLastErr
End Property
Private Function ValidateMetaFile(intNumF As Integer, intFN As Integer) As Boolean
' checks the integrity of the data.

Select Case intNumF
Case 0
    If LOF(intFN) = 0 Then
        ValidateMetaFile = True
    Else
        ValidateMetaFile = False
    End If
Case Is <> 0
    If LOF(intFN) <> 6 + (28 * CLng(intNumF)) Then
        ValidateMetaFile = False
    Else
        ValidateMetaFile = True
    End If
End Select

End Function
Public Function Initialize(ByVal strRepFile As String, ByVal strMetaFile As String) As Boolean
'creates the resource files and then saves the fully qualified
'file names in registry.
On Error GoTo ErrHandler
If m_blnStatus Then Exit Function
If strRepFile <> "" And strMetaFile <> "" Then
    m_strMetaFile = strMetaFile
    m_strRepFile = strRepFile
    
    If FileExists(m_strMetaFile) Then
        Kill m_strMetaFile
    End If
    If FileExists(m_strRepFile) Then
        Kill m_strRepFile
    End If

    OpenFile
    SaveSetting "prjTransferData", "Startup", "Path1", m_strMetaFile
    SaveSetting "prjTransferData", "Startup", "Path2", m_strRepFile
    Initialize = True
    
Else
    Err.Raise vbObjectError + 2010, , "Creating the repository failed."
    Initialize = False
End If
Exit Function
ErrHandler:
m_strLastErr = "Error " & Err.Number & "; prjTransferData.clsFile; " & Err.Description
Initialize = False
End Function
Public Property Get Status() As Boolean
'returns True if the repository exists.
Status = m_blnStatus
End Property
